<?php
/**
 * Tapulous Twinkle image discoverer and embedder
 *
 *  Takes a Twitter message, searches for an embedded Twinkle image, and
 *  replaces the URL to the image with an image tag, wrapped in a div tag,
 *  and scaled to thumbnail size.
 *
 *  Caches results in a database table for speed.
 *
 *  Usage:
 *      $message = TwinkleImageUtil::processContent($message, 
 *                                                  $cacheDatabaseHandle,
 *                                                  $options);
 *
 *      Options: An associative array with specified options. Defaults are:
 *
 *                  divClass                = "twinkle_image"
 *                  thumbnailDimensions     = array(35,35)
 *                  thumbnailDirectory      = "twinkle-thumbs"
 *                  thumbnailURLPrefix      = ""
 *                  cacheTable              = "twinklecache"
 *                  lightboxMode            = false
 *
 *
 *  Copyright 2008 Michael Tyson <mike@tyson.id.au>, Tzi Software
 *  Version 0.1, 8 September 2008
 *  GPL License
 */

class TwinkleImageUtil {

    function processContent($content, 
                            $cacheDatabaseHandle,
                            $options = array()) {
        

        $defaults = array(
                                "divClass"              => "twinkle_image", 
                                "thumbnailDimensions"   => array(35,35), 
                                "thumbnailDirectory"    => "twinkle-thumbs", 
                                "thumbnailURLPrefix"    => "", 
                                "cacheTable"            => "twinklecache",
                                "lightboxMode"          => false);
        
        $options = array_merge($defaults, $options);
        
        if ( strpos($content, "http://snipurl.com") === false ) {
            // There's nothing here. Just return the content
            return $content;
        }
        
        $id = md5($content);
        
        // Try to find cached version
        if ( ($info = TwinkleImageUtil::findInCache($cacheDatabaseHandle, $options["cacheTable"], $id)) !== false ) {
            
            if ( !$info ) {
                // No image here
                return $content;
            }
            
            // See if thumbnail exists
            if ( file_exists($options["thumbnailDirectory"]."/".$id.".jpg") ) {
                // Thumbnail's there - we're done
                return $info["content"];
            }
        }
        
        
        if ( !$info ) {
            // No cached version exists. Try to discover a twinkle image in the content
            if ( !($info = TwinkleImageUtil::findTwinkleImageFromContent($content)) ) {
                // Didn't find a twinkle image. Cache result
                TwinkleImageUtil::saveCache($cacheDatabaseHandle, $options["cacheTable"], $id, $info);
                
                // Return content as-is
                return $content;
            }
        }
                
        // Found an image; create a thumbnail
        if ( ($hasThumb=TwinkleImageUtil::createThumbnail($id, $info /* In/Out param */, $options["thumbnailDimensions"], $options["thumbnailDirectory"])) ) {
            // Succeeded - create url to thumbnail
            $thumbUrl = $options["thumbnailURLPrefix"].$options["thumbnailDirectory"]."/".$id.".jpg";
        } else {
            // Failed making thumbnail - just link to original
            $thumbUrl = $imageUrl;
            $info["thumbnailDimensions"] = $options["thumbnailDimensions"];
        }
        
        // Create tag
        $alt = htmlspecialchars(trim(preg_replace("@http://snipurl.com/[0-9a-zA-Z]+@", "", $content)));
        $tag = "<div class=\"".$options["divClass"]."\">".
                ($options["lightboxMode"] ? "<a href=\"".$info["imageUrl"]."\" rel=\"lightbox[twinkle]\" title=\"".$alt."\">" : "<a href=\"".$info["linkUrl"]."\" title=\"".$alt."\">").
                "<img src=\"".$thumbUrl."\" ".($info["thumbnailDimensions"] ? "width=\"".$info["thumbnailDimensions"][0]."\" height=\"".$info["thumbnailDimensions"][1]."\"" : "").
                    "alt=\"".$alt."\" />".
                "</a></div>";
                
        // Insert tag into content
        $info["content"] = preg_replace("@http://snipurl.com/[0-9a-zA-Z]+@", "$tag", $content);
        
        // Save to cache
        TwinkleImageUtil::saveCache($cacheDatabaseHandle, $options["cacheTable"], $id, $info);
        
        return $info["content"];
    }




    function findTwinkleImageFromContent($content) {
    
        if ( !preg_match("|http://snipurl.com/([0-9a-zA-Z]+)|", $content, $matches, PREG_OFFSET_CAPTURE) ) 
            return false;
    
        $snipId = $matches[1][0];
    
        if ( !($fd = @fopen("http://snipurl.com/resolveurl?id=".$snipId, "r")) )
            return false;
    
        $url = "";
        while ( !feof($fd) )
            $url .= fgets($fd);
        fclose($fd);
    
        if ( !preg_match("|http://twinkle.tapulous.com/index.php\?hash=[0-9a-zA-Z]+|", $url) )
            return false;
        
        if ( !($fd = @fopen(trim($url), "r")) )
            return false;
        
        $data = "";
        while ( !feof($fd) )
            $data .= fgets($fd);
        fclose($fd);
    
        if ( !preg_match("@<img[^>]+?src=\"(http://[^\"]+.jpg)\"@", $data, $datamatches) )
            return;
    
        $info = array();
        $info["imageUrl"] = $datamatches[1];
        $info["linkUrl"] = $url;
        
        return $info;
    }
    
    function createThumbnail($id, &$info, $thumbnailDimensions, $thumbnailDirectory) {
        
        if ( !file_exists($thumbnailDirectory) ) {
            mkdir($thumbnailDirectory, 0755);
        }
        
        if ( !file_exists($thumbnailDirectory) ) {
            return false;
        }
        
        $output = $thumbnailDirectory."/".$id.".jpg";
        
        if ( file_exists($output) ) {
            return true;
        }
    
        if ( !function_exists("imagecreatefromjpeg") ) {
            // GD2 is not installed: try downloading the image, then using imagemagick's convert
            if ( !($fd = @fopen($info["imageUrl"], "r")) ) {
                // Couldn't download
                return false;
            }
            
            $tmpImagePath = "twinkleimage-tmp.jpg";
            
            if ( !($ofd = fopen($tmpImagePath, "w")) ) {
                // Couldn't open output image
                return false;
            }
            
            while ( !feof($fd) )
                fwrite($ofd, fgets($fd));
            fclose($ofd);
            fclose($fd);
            
            if ( trim(shell_exec("convert ".escapeshellarg($tmpImagePath)." -resize ".escapeshellarg($thumbnailDimensions[0]."x".$thumbnailDimensions[1])." ".escapeshellarg($output)." && echo success")) != "success" ||
                !file_exists($output) ) {
                @unlink($output);
                return false;
            }
            @unlink($tmpImagePath);
            
            return true;
        }
        
        $image = @imagecreatefromjpeg($info["imageUrl"]);
        
        if ( !$image ) {
            // Failed opening the image
            return false;
        }
        
        $scaledImage = TwinkleImageUtil::scaleImage($image, $thumbnailDimensions);
        
        if ( !$scaledImage ) {
            // Failed scaling the image
            return false;
        }
        
        $info["thumbnailDimensions"] = array(imagesx($scaledImage), imagesy($scaledImage));
        

        if ( !imagejpeg($scaledImage, $output) ) {
            return false;
        }
        
        imagedestroy($image);
        imagedestroy($scaledImage);
        
        return true;
    }
     
    function scaleImage($image, $dimensions)
    {
        // Read original image dimensions
        $orig_w=imagesx($image);
        $orig_h=imagesy($image);
        
        list($width,$height) = $dimensions;

        // Scale proportionately to within height/width
        if ($orig_w>$width || $orig_h>$height)
        {
           $img_w=$width;
           $img_h=$height;
           if ($img_w/$orig_w*$orig_h>$img_h)
              $img_w=round($img_h*$orig_w/$orig_h);
           else
              $img_h=round($img_w*$orig_h/$orig_w);
        } 
        else
        {
           $img_w=$orig_w;
           $img_h=$orig_h;
        }
        
        // Create scaled image
        $newImage = imagecreatetruecolor($img_w,$img_h);
 
        if ( !$newImage )
           return false;
 
        imagecopyresampled($newImage,$image,0,0,0,0,$img_w,$img_h,$orig_w,$orig_h);
         
        return $newImage;
    }
    
    function findInCache($cacheDatabaseHandle, $cacheTable, $id) {
        $result = mysql_query("SELECT * FROM `".$cacheTable."` WHERE identifier = '".mysql_escape_string($id)."'");
        if ( !$result || !mysql_num_rows($result) ) return false;
        
        $obj = mysql_fetch_object($result);
        $info = @unserialize($obj->info);
        
        if ( !$info ) 
            $info = array();
        if ( $obj->content )
            $info["content"] = $obj->content;
        
        return $info;
    }
    
    function saveCache($cacheDatabaseHandle, $cacheTable, $id, $info) {
        if ( !TwinkleImageUtil::createCache($cacheDatabaseHandle, $cacheTable) )
            return false;
        
        return mysql_query(
            "INSERT INTO `".$cacheTable."` (`identifier`, `content`, `info`) VALUES (".
                "'".mysql_escape_string($id)."',".
                "'".mysql_escape_string($info["content"])."',".
                "'".mysql_escape_string(serialize($info))."')", $cacheDatabaseHandle);
    }
    
    function createCache($cacheDatabaseHandle, $cacheTable) {
        return mysql_query(
            "CREATE TABLE IF NOT EXISTS `".$cacheTable."` (
              `id` int(11) NOT NULL auto_increment,
              `identifier` varchar(32) default NULL,
              `content` text,
              `info` text,
              PRIMARY KEY  (`id`),
              UNIQUE KEY `identifier` (`identifier`)
            ) ENGINE=MyISAM DEFAULT CHARSET=latin1",
            $cacheDatabaseHandle);
    }
}

?>